@@ -20,14 +20,13 @@ using Beagle.Util;
 namespace ImLogViewer {
 
 	public class GaimLogViewer {
-		[Widget] Window logwindow;
+		[Widget] Dialog log_dialog;
 		[Widget] TreeView timelinetree;
 		[Widget] ScrolledWindow scrollwindow;
-		[Widget] Button searchbutton;
-		[Widget] Label title;
-		[Widget] Entry search;
+		[Widget] Label time_title;
+		[Widget] Entry search_entry;
+		[Widget] Label find_label;
 		[Widget] TextView conversation;
-		[Widget] Image buddyicon;
 		
 		private TreeStore treeStore;
 		private CellRendererText renderer;
@@ -50,6 +49,9 @@ namespace ImLogViewer {
 			} else if (File.Exists (path)) {
 				log_path = Path.GetDirectoryName (path);
 				first_selected_log = path;
+			} else {
+				Console.WriteLine ("Log path doesn't exist - {0}", path);
+				return;
 			}
 
 			timeline = new Timeline ();
@@ -61,11 +63,8 @@ namespace ImLogViewer {
 
 		private void SetTitle (DateTime dt)
 		{
-			string str = "<b>Conversation with " + speaking_to;
 			if (dt.Ticks > 0)
-				str += ": " + StringFu.DateTimeToPrettyString (dt);
-			str += "</b>";
-			title.Markup = str;
+				time_title.Markup = String.Format ("<b>{0}</b>",StringFu.DateTimeToPrettyString (dt));
 		}
 		
 		Gtk.AccelGroup accel_group;
@@ -75,37 +74,42 @@ namespace ImLogViewer {
 		{
 			Application.Init();
 			
-			Glade.XML gxml = new Glade.XML (null, "ImLogViewer.glade", "logwindow", null);
+			Glade.XML gxml = new Glade.XML (null, "ImLogViewer.glade", "log_dialog", null);
 			gxml.Autoconnect (this);
 
+			log_dialog.Response += new ResponseHandler (OnWindowResponse);
+
+			// FIXME: Hide the find bar until further notice. We want highlighing and queries using Beagle
+			search_entry.Visible = false;
+			find_label.Visible = false;
+
 			accel_group = new Gtk.AccelGroup ();
-			logwindow.AddAccelGroup (accel_group);
+			log_dialog.AddAccelGroup (accel_group);
 			global_keys = new GlobalKeybinder (accel_group);
 
-			// Buddy icon
+			// Find the buddy
 			GaimBuddyListReader list = new GaimBuddyListReader ();
 			ImBuddy buddy = list.Search (speaker);
-			if (buddy != null && buddy.BuddyIconLocation != "") {
-				//Console.WriteLine ("Found buddy info for {0}, icon at {1}", buddy.BuddyAccountName, buddy.BuddyIconLocation);
-				string homedir = Environment.GetEnvironmentVariable ("HOME");
-				string fullpath = Path.Combine (homedir, ".gaim");
-				fullpath = Path.Combine (fullpath, "icons");
-				fullpath = Path.Combine (fullpath, buddy.BuddyIconLocation);
-				buddyicon.Pixbuf = new Gdk.Pixbuf (fullpath).ScaleSimple (32, 32, Gdk.InterpType.Bilinear);
+			
+			if (buddy != null && buddy.Alias != "") {
+				log_dialog.Title = String.Format ("Conversations with {0}", buddy.Alias);
 			} else {
-				buddyicon.Visible = false;
+				if (speaker.EndsWith (".chat"))
+					log_dialog.Title = String.Format ("Conversations in {0}", speaker.Replace (".chat",""));
+				else
+					log_dialog.Title = String.Format ("Conversation with {0}", speaker);
 			}
-				
+
 			SetTitle (new DateTime ());
 
 			// Close window (Ctrl-W)
-			global_keys.AddAccelerator (new EventHandler (this.HideWindow),
+			global_keys.AddAccelerator (new EventHandler (this.OnWindowClose),
 						    (uint) Gdk.Key.w, 
 						    Gdk.ModifierType.ControlMask,
 						    Gtk.AccelFlags.Visible);
 
 			// Close window (Escape)
-			global_keys.AddAccelerator (new EventHandler (this.HideWindow),
+			global_keys.AddAccelerator (new EventHandler (this.OnWindowClose),
 						    (uint) Gdk.Key.Escape, 
 						    0,
 						    Gtk.AccelFlags.Visible);
@@ -137,9 +141,7 @@ namespace ImLogViewer {
 			PopulateTimelineWidget ();
 
 			timelinetree.Selection.Changed += OnConversationSelected; 
-			search.Activated += OnSearch;
-			search.Changed += OnTypeAhead;
-			searchbutton.Activated += OnSearch;
+			search_entry.Changed += OnTypeAhead;
 
 			if (have_first_selection_iter)
 				timelinetree.Selection.SelectIter (first_selection_iter);
@@ -170,6 +172,7 @@ namespace ImLogViewer {
 
 		private void UpdateTimelineTree ()
 		{
+			//FIXME: Run this in a thread
 			treeStore = new TreeStore(new Type[] {typeof(string), typeof(string), typeof(object)});
 			timelinetree.Model = this.treeStore;
 			PopulateTimelineWidget ();
@@ -217,22 +220,11 @@ namespace ImLogViewer {
 			timelinetree.ExpandAll();
 		}
 
-		private class ReverseLogComparer : IComparer {
-			
-			public int Compare (object x, object y)
-			{
-				return ((ImLog) y).StartTime.CompareTo (((ImLog) x).StartTime);
-			}
-		}
-
-		static ReverseLogComparer rev_cmp = new ReverseLogComparer ();
-
 		private void PopulateTimeline (TreeIter parent, ArrayList list, string dateformat)
 		{
-			list.Sort (rev_cmp);
 			foreach (ImLog log in list) {
-				if (search.Text != null && search.Text != "")
-					if (!LogContainsString (log, search.Text))
+				if (search_entry.Text != null && search_entry.Text != "")
+					if (!LogContainsString (log, search_entry.Text))
 						continue;
 					
 				string date_str = log.StartTime.ToString (dateformat);
@@ -245,12 +237,9 @@ namespace ImLogViewer {
 			}
 		}
 		
-		private void IndexLogs () {
-		       	string [] files = Directory.GetFiles (log_path);
-			
-			foreach (string file in files) {
-
-
+		private void IndexLogs ()
+		{
+			foreach (string file in Directory.GetFiles (log_path)) {
 				ICollection logs = GaimLog.ScanLog (new FileInfo (file));
 				
 				foreach (ImLog log in logs) {
@@ -284,19 +273,14 @@ namespace ImLogViewer {
 			}
 		}
 
-		private void HideWindow (object o, EventArgs args)
+		private void OnWindowClose (object o, EventArgs args)
 		{
 			Application.Quit ();
 		}
 
-		private void OnWindowDeleteEvent (object o, DeleteEventArgs args)
+		private void OnWindowResponse (object o, ResponseArgs args)
 		{
 			Application.Quit ();
-			args.RetVal = true;
-		}
-
-		private void OnSearch (object o, EventArgs args)
-		{
 		}
 
 		private void OnTypeAhead (object o, EventArgs args)
@@ -321,7 +305,7 @@ namespace ImLogViewer {
 				new GaimLogViewer (args [0]);
 			} else {
 				Console.WriteLine ("USAGE: beagle-imlogviewer " +
-						   "/home/lukas/.gaim/logs/msn/joe@hotmail.com/jane@hotmail.com");
+						   "/home/lukas/.gaim/logs/msn/joe@joe.com/jane@love.com");
 			}
 		}
 	}
