@@ -56,7 +56,16 @@ namespace Beagle.Util {
 		private void DoPull (int neededSize)
 		{
 			while (! done && pullBuffer.Length < neededSize) {
-				string str = pull ();
+
+				string str = null;
+				try { 
+					str = pull ();
+				} catch (Exception ex) {
+					// If we catch an exception, str will still be null
+					// and thus the code below will set done to true.
+					Logger.Log.Debug ("Caught exception pulling text from {0}", pull);
+					Logger.Log.Debug (ex);
+				}
 
 				if (str != null) {
 					pullBuffer.Append (str);
