@@ -33,11 +33,13 @@ using System.Threading;
 using System.Xml;
 
 using Beagle.Util;
+using Beagle.Daemon;
+
 using Camel = Beagle.Util.Camel;
 using GConf;
 using Mono.Posix;
 
-namespace Beagle.Daemon {
+namespace Beagle.Daemon.EvolutionMailDriver {
 
 	public abstract class EvolutionMailIndexableGenerator : IIndexableGenerator {
 
@@ -112,6 +114,11 @@ namespace Beagle.Daemon {
 			}
 		}
 
+		static public bool SupportedContentType (GMime.ContentType type)
+		{
+			return type.Type == "text" || type.Type == "Text" || type.Type == "TEXT";
+		}
+
 		protected class PartHandler {
 			private MultiReader reader = null;
 			private int depth = 0;
@@ -130,16 +137,34 @@ namespace Beagle.Daemon {
 				if (part is GMime.MessagePart) {
 					GMime.MessagePart msg_part = (GMime.MessagePart) part;
 
-					msg_part.Message.ForeachPart (new GMime.PartFunc (this.OnEachPart));
+					using (GMime.Message message = msg_part.Message) {
+						using (GMime.Object subpart = message.MimePart)
+							this.OnEachPart (subpart);
+					}
 				} else if (part is GMime.Multipart) {
 					GMime.Multipart multipart = (GMime.Multipart) part;
 
-					multipart.ForeachPart (new GMime.PartFunc (this.OnEachPart));
-				} else {
+					int num_parts = multipart.Number;
+					for (int i = 0; i < num_parts; i++) {
+						using (GMime.Object subpart = multipart.GetPart (i))
+							this.OnEachPart (subpart);
+					}
+				} else if (SupportedContentType (part.ContentType)) {
 					MemoryStream stream = new MemoryStream (part.GetData ());
 					StreamReader reader = new StreamReader (stream);
+
+					// If this is the first part, we need to skip past the
+					// message headers --- we want to store that data in
+					// properties, not index it as text.
+					if (this.reader.Count == 0) {
+						string line;
+						do {
+							line = reader.ReadLine ();
+						} while (line != null && line != "");
+					}
+
 					this.reader.Add (reader);
-				}
+				} 
 
 				--depth;
 			}
@@ -151,7 +176,8 @@ namespace Beagle.Daemon {
 				// from a single indexable.
 				PartHandler handler = new PartHandler ();
 				//message.ForeachPart (new GMime.PartFunc (handler.OnEachPart));
-				handler.OnEachPart (message.MimePart);
+				using (GMime.Object mime_part = message.MimePart)
+					handler.OnEachPart (mime_part);
 				return handler.reader;
 			}
 
@@ -159,7 +185,7 @@ namespace Beagle.Daemon {
 
 		protected static Stream ReadAppData (string name)
 		{
-			string path = Path.Combine (Path.Combine (PathFinder.RootDir, "MailIndex"), name);
+			string path = Path.Combine (Path.Combine (PathFinder.StorageDir, "MailIndex"), name);
 			return new FileStream (path, System.IO.FileMode.Open, FileAccess.Read, FileShare.Read);
 		}
 
@@ -181,14 +207,14 @@ namespace Beagle.Daemon {
 
 		protected static Stream WriteAppData (string name)
 		{
-			string path = Path.Combine (Path.Combine (PathFinder.RootDir, "MailIndex"), name);
+			string path = Path.Combine (Path.Combine (PathFinder.StorageDir, "MailIndex"), name);
 			return new FileStream (path, System.IO.FileMode.Create, FileAccess.Write, FileShare.None);
 		}
 
 		protected static void WriteAppDataLine (string name, string line)
 		{
 			if (line == null) {
-				string path = Path.Combine (Path.Combine (PathFinder.RootDir, "MailIndex"), name);
+				string path = Path.Combine (Path.Combine (PathFinder.StorageDir, "MailIndex"), name);
 
 				if (File.Exists (path))
 					File.Delete (path);
@@ -220,6 +246,11 @@ namespace Beagle.Daemon {
 			else
 				return false;
 		}
+
+		public override int GetHashCode ()
+		{
+			return this.CrawlFile.FullName.GetHashCode ();
+		}
 	}
 
 	public class EvolutionMailIndexableGeneratorMbox : EvolutionMailIndexableGenerator {
@@ -243,14 +274,14 @@ namespace Beagle.Daemon {
 			while (di != null) {
 				// Evo uses ".sbd" as the extension on a folder
 				if (di.Extension == ".sbd")
-					folder_name = Path.Combine (folder_name, Path.GetFileNameWithoutExtension (di.Name));
+					folder_name = Path.Combine (Path.GetFileNameWithoutExtension (di.Name), folder_name);
 				else
 					break;
 
 				di = di.Parent;
 			}
 
-			return Path.Combine (folder_name, Path.GetFileNameWithoutExtension (file_info.Name));
+			return Path.Combine (folder_name, file_info.Name);
 		}
 
 		protected override bool Setup ()
@@ -287,8 +318,16 @@ namespace Beagle.Daemon {
 
 			if (this.mbox_fd < 0) {
 				Logger.Log.Debug ("opening mbox {0}", this.mbox_info.Name);
+
+				try {
+					InitializeGMime ();
+				} catch (Exception e) {
+					Logger.Log.Warn ("Caught exception trying to initalize gmime:");
+					Logger.Log.Warn (e);
+					return false;
+				}
+
 				this.mbox_fd = Syscall.open (this.mbox_info.FullName, OpenFlags.O_RDONLY);
-				InitializeGMime ();
 				this.mbox_stream = new GMime.StreamFs (this.mbox_fd);
 				this.mbox_stream.Seek ((int) this.MboxLastOffset);
 				this.mbox_parser = new GMime.Parser (this.mbox_stream);
@@ -301,7 +340,9 @@ namespace Beagle.Daemon {
 				this.mbox_stream.Close ();
 
 				this.mbox_fd = -1;
+				this.mbox_stream.Dispose ();
 				this.mbox_stream = null;
+				this.mbox_parser.Dispose ();
 				this.mbox_parser = null;
 				
 				EvolutionMailQueryable.log.Debug ("{0}: Finished indexing {1} messages",
@@ -317,37 +358,37 @@ namespace Beagle.Daemon {
 
 		public override Indexable GetNextIndexable ()
 		{
-			GMime.Message message = this.mbox_parser.ConstructMessage ();
+			using (GMime.Message message = this.mbox_parser.ConstructMessage ()) {
+				++this.count;
 
-			++this.count;
-
-			// Work around what I think is a bug in GMime: If you
-			// have a zero-byte file or seek to the end of a
-			// file, parser.Eos () will return true until it
-			// actually tries to read something off the wire.
-			// Since parser.ConstructMessage() always returns a
-			// message (which may also be a bug), we'll often get
-			// one empty message which we need to deal with here.
-			//
-			// Check if its empty by seeing if the Headers
-			// property is null or empty.
-			if (message.Headers == null || message.Headers == "")
-				return null;
-
-			string x_evolution = message.GetHeader ("X-Evolution");
-			if (x_evolution == null || x_evolution == "") {
-				EvolutionMailQueryable.log.Info ("{0}: Message at offset {1} has no X-Evolution header!",
-								 this.folder_name, this.mbox_parser.FromOffset);
-				return null;
+				// Work around what I think is a bug in GMime: If you
+				// have a zero-byte file or seek to the end of a
+				// file, parser.Eos () will return true until it
+				// actually tries to read something off the wire.
+				// Since parser.ConstructMessage() always returns a
+				// message (which may also be a bug), we'll often get
+				// one empty message which we need to deal with here.
+				//
+				// Check if its empty by seeing if the Headers
+				// property is null or empty.
+				if (message.Headers == null || message.Headers == "")
+					return null;
+				
+				string x_evolution = message.GetHeader ("X-Evolution");
+				if (x_evolution == null || x_evolution == "") {
+					EvolutionMailQueryable.log.Info ("{0}: Message at offset {1} has no X-Evolution header!",
+									 this.folder_name, this.mbox_parser.FromOffset);
+					return null;
+				}
+				
+				string uid_str = x_evolution.Substring (0, x_evolution.IndexOf ('-'));
+				string uid = Convert.ToUInt32 (uid_str, 16).ToString (); // ugh.
+				
+				Indexable indexable = this.GMimeMessageToIndexable (uid, message);
+				++this.indexed_count;
+				
+				return indexable;
 			}
-
-			string uid_str = x_evolution.Substring (0, x_evolution.IndexOf ('-'));
-			string uid = Convert.ToUInt32 (uid_str, 16).ToString (); // ugh.
-
-			Indexable indexable = this.GMimeMessageToIndexable (uid, message);
-			++this.indexed_count;
-
-			return indexable;
 		}
 
 		private Indexable GMimeMessageToIndexable (string uid, GMime.Message message)
@@ -359,13 +400,15 @@ namespace Beagle.Daemon {
 			indexable.Type = "MailMessage";
 			indexable.MimeType = "text/plain"; // FIXME
 
-			indexable.AddProperty (Property.NewKeyword ("dc:title",       message.Subject));
+			indexable.AddProperty (Property.New ("fixme:client", "evolution"));
+
+			indexable.AddProperty (Property.NewKeyword ("dc:title",       GMime.Utils.HeaderDecodePhrase (message.Subject)));
 
 			indexable.AddProperty (Property.NewKeyword ("fixme:account",  "Local"));
                         indexable.AddProperty (Property.NewKeyword ("fixme:folder",   this.folder_name));
-			indexable.AddProperty (Property.NewKeyword ("fixme:subject",  message.Subject));
+			indexable.AddProperty (Property.NewKeyword ("fixme:subject",  GMime.Utils.HeaderDecodePhrase (message.Subject)));
                         indexable.AddProperty (Property.NewKeyword ("fixme:to",       message.GetRecipientsAsString (GMime.Message.RecipientType.To)));
-			indexable.AddProperty (Property.NewKeyword ("fixme:from",     message.Sender));
+			indexable.AddProperty (Property.NewKeyword ("fixme:from",     GMime.Utils.HeaderDecodePhrase (message.Sender)));
 			indexable.AddProperty (Property.NewKeyword ("fixme:cc",       message.GetRecipientsAsString (GMime.Message.RecipientType.Cc)));
 
 			if (this.folder_name == "Sent")
@@ -378,14 +421,14 @@ namespace Beagle.Daemon {
 
 			if (list_id != null) {
 				// FIXME: Might need some additional parsing.
-				indexable.AddProperty (Property.NewKeyword ("fixme:mlist", list_id));
+				indexable.AddProperty (Property.NewKeyword ("fixme:mlist", GMime.Utils.HeaderDecodePhrase (list_id)));
 			}
 
 			if (this.folder_name == "Sent")
 				indexable.AddProperty (Property.NewDate ("fixme:sentdate", message.Date));
 			else
 				indexable.AddProperty (Property.NewDate ("fixme:received", message.Date));
-
+			
 #if false
 			// FIXME - XXX
                         indexable.AddProperty (Property.NewKeyword ("fixme:flags",    messageInfo.flags));
@@ -414,7 +457,8 @@ namespace Beagle.Daemon {
 			if (messageInfo.IsAnsweredAll)
 				indexable.AddProperty (Property.NewFlag ("fixme:isAnsweredAll"));
 #endif
-	                indexable.SetTextReader (PartHandler.GetReader (message));
+
+			indexable.SetTextReader (PartHandler.GetReader (message));
 
 			return indexable;
 		}
@@ -482,7 +526,17 @@ namespace Beagle.Daemon {
 			string imap_start = dir_name.Substring (imap_start_idx);
 			this.imap_name = imap_start.Substring (0, imap_start.IndexOf ('/'));
 
-			this.accounts = (ICollection) GConfThreadHelper.Get ("/apps/evolution/mail/accounts");
+			try {
+				this.accounts = (ICollection) GConfThreadHelper.Get ("/apps/evolution/mail/accounts");
+			} catch (Exception ex) {
+				EvolutionMailQueryable.log.Warn ("Caught exception in Setup(): " + ex.Message);
+				EvolutionMailQueryable.log.Warn ("There are no configured evolution accounts, ignoring {0}", this.imap_name);
+				return false;
+			}
+
+			// This should only happen if we shut down while waiting for the GConf results to come back.
+			if (this.accounts == null)
+				return false;
 
 			foreach (string xml in this.accounts) {
 				XmlDocument xmlDoc = new XmlDocument ();
@@ -599,8 +653,15 @@ namespace Beagle.Daemon {
 				}
 			}
 
-			if (this.summary == null)
-				this.summary = Camel.Summary.load (this.summary_info.FullName);
+			if (this.summary == null) {
+				try {
+					this.summary = Camel.Summary.load (this.summary_info.FullName);
+				} catch (Exception e) {
+					EvolutionMailQueryable.log.Warn ("Unable to index {0}: {1}", this.folder_name,
+									 e.Message);
+					return false;
+				}
+			}
 
 			if (this.summary_enumerator == null)
 				this.summary_enumerator = this.summary.GetEnumerator ();
@@ -608,10 +669,11 @@ namespace Beagle.Daemon {
 			if (this.summary_enumerator.MoveNext ())
 				return true;
 
-			// FIXME: This is kind of a hack, but it's the only way with the IndexableGenerator
-			// to handle our removals.
 			foreach (string uid in this.deleted_list) {
 				Uri uri = EvolutionMailQueryable.EmailUri (this.account_name, this.folder_name, uid);
+
+				// FIXME: This is kind of a hack, but it's the only way
+				// with the IndexableGenerator to handle our removals.
 				Scheduler.Task task = this.Queryable.NewRemoveTask (uri);
 				task.Priority = Scheduler.Priority.Immediate;
 				this.Queryable.ThisScheduler.Add (task);
@@ -630,19 +692,26 @@ namespace Beagle.Daemon {
 
 		private MultiReader GetMessageData (string file)
 		{
-			int fd;
-
-			fd = Syscall.open (file, OpenFlags.O_RDONLY);
-
-			InitializeGMime ();
+			try {
+				InitializeGMime ();
+			} catch (Exception e) {
+				Logger.Log.Warn ("Caught exception trying to initialize gmime:");
+				Logger.Log.Warn (e);
+				return null;
+			}
 
+			int fd = Syscall.open (file, OpenFlags.O_RDONLY);
 			GMime.StreamFs stream = new GMime.StreamFs (fd);
 			GMime.Parser parser = new GMime.Parser (stream);
-			GMime.Message message = parser.ConstructMessage ();
 
-			MultiReader reader = PartHandler.GetReader (message);
+			MultiReader reader = null;
+
+			using (GMime.Message message = parser.ConstructMessage ())
+				reader = PartHandler.GetReader (message);
 
 			stream.Close ();
+			stream.Dispose ();
+			parser.Dispose ();
 
 			return reader;
 		}
@@ -674,9 +743,14 @@ namespace Beagle.Daemon {
 			return indexable;
 		}
 
+		private Uri CamelMessageUri (Camel.MessageInfo message_info)
+		{
+			return EvolutionMailQueryable.EmailUri (this.account_name, this.folder_name, message_info.uid);			
+		}
+
 		private Indexable CamelMessageToIndexable (Camel.MessageInfo messageInfo, TextReader msgReader)
 		{
-			System.Uri uri = EvolutionMailQueryable.EmailUri (this.account_name, this.folder_name, messageInfo.uid);
+			Uri uri = CamelMessageUri (messageInfo);
 			Indexable indexable = new Indexable (uri);
 
 			indexable.Timestamp = messageInfo.Date;
