@@ -37,20 +37,6 @@ namespace Beagle.Daemon {
 
 	public class Filter {
 
-		public enum ContentType {
-			Text,
-			WhiteSpace,
-			StructuralBreak
-		}
-
-		public delegate void GotPropertyHandler (Property prop);
-		public delegate void GotContentHandler  (ContentType type, string content);
-
-		public event GotPropertyHandler GotPropetyEvent;
-		public event GotContentHandler  GotContentEvent;
-
-		//////////////////////////
-
 		// Derived classes always must have a constructor that
 		// takes no arguments.
 		public Filter () { }
@@ -159,11 +145,8 @@ namespace Beagle.Daemon {
 		
 		public void HotDown ()
 		{
-			if (hotCount > 0) {
+			if (hotCount > 0)
 				--hotCount;
-				if (hotCount == 0)
-					AppendWhiteSpace ();
-			}
 		}
 
 		public bool IsHot {
@@ -177,7 +160,8 @@ namespace Beagle.Daemon {
 
 		public void FreezeDown ()
 		{
-			--freezeCount;
+			if (freezeCount > 0)
+				--freezeCount;
 		}
 
 		public bool IsFrozen {
@@ -229,8 +213,12 @@ namespace Beagle.Daemon {
 					str = str.Replace ("\n", " ");
 
 				textPool.Add (str);
+
+				// For Hot pool, Add the "whitespace"
+				// while adding text to it. This way
+				// we can avoid extra spaces.
 				if (IsHot)
-					hotPool.Add (str);
+					hotPool.Add (str.Trim()+" ");
 
 				int pool_size = 0;
 				foreach (string x in textPool)
@@ -277,9 +265,6 @@ namespace Beagle.Daemon {
 					snippetWriter.Write (" ");
 				last_was_structural_break = false;
 			}
-			if (NeedsWhiteSpace (hotPool))
-				hotPool.Add (" ");
-			
 		}
 
 		public void AddProperty (Property prop)
@@ -294,6 +279,10 @@ namespace Beagle.Daemon {
 				snippetWriter.WriteLine ();
 				last_was_structural_break = true;
 			}
+			// When adding a "newline" to the textCache, we need to 
+			// append a "Whitespace" to the text pool.
+			if (NeedsWhiteSpace (textPool))
+				textPool.Add (" ");
 		}
 
 		//////////////////////////
@@ -485,14 +474,27 @@ namespace Beagle.Daemon {
 			return null;
 		}
 
+		private string PullTextCarefully (ArrayList array)
+		{
+			string text = null;
+			try {	
+				text = PullFromArray (array);
+			} catch (Exception ex) {
+				Logger.Log.Debug ("Caught exception while pulling text in filter '{0}'", Name);
+				Logger.Log.Debug (ex);
+			}
+			return text;
+
+		}
+
 		private string PullText ()
 		{
-			return PullFromArray (textPool);
+			return PullTextCarefully (textPool);
 		}
 
 		private string PullHotText ()
 		{
-			return PullFromArray (hotPool);
+			return PullTextCarefully (hotPool);
 		}
 
 		public TextReader GetTextReader ()
@@ -504,8 +506,7 @@ namespace Beagle.Daemon {
 
 		public TextReader GetHotTextReader ()
 		{
-			return null;
-			//return new PullingReader (new PullingReader.Pull (PullHotText));
+			return new PullingReader (new PullingReader.Pull (PullHotText));
 		}
 
 		public IEnumerable Properties {
