@@ -53,6 +53,7 @@ namespace Beagle.Daemon {
 	public class LuceneDriver : IIndexer {
 
 		public delegate bool UriFilter (Uri uri);
+		public delegate Uri UriRemapper (Uri uri);
 		public delegate double RelevancyMultiplier (Hit hit);
 
 		public event IIndexerChangedHandler ChangedEvent;
@@ -69,7 +70,11 @@ namespace Beagle.Daemon {
 		// 7: incremented to force a re-index after our upgrade to lucene 1.4
 		//    (in theory the file formats are compatible, we are seeing 'term
 		//    out of order' exceptions in some cases)
-		private const int VERSION = 7;
+		// 8: another forced re-index, this time because of massive changes
+		//    in the file system backend (it would be nice to have per-backend
+		//    versioning so that we didn't have to purge all indexes just
+		//    because one changed)
+		private const int VERSION = 8;
 
 		private string top_dir;
 		private Hashtable pending_by_uri = UriFu.NewHashtable ();
@@ -192,8 +197,7 @@ namespace Beagle.Daemon {
 			}
 
 			Lucene.Net.Store.FSDirectory store;
-			store = Lucene.Net.Store.FSDirectory.GetDirectory (indexDir, false);
-			store.TempDirectoryName = lockDir;
+			store = Lucene.Net.Store.FSDirectory.GetDirectory (indexDir, lockDir, false);
 			ourStore = store;
 			ourStorePath = indexDir;
 
@@ -236,6 +240,8 @@ namespace Beagle.Daemon {
 		// Public Indexing API
 		//
 
+		static object [] empty_collection = new object [0];
+
 		public void Add (Indexable indexable)
 		{
 			Uri uri = indexable.Uri;
@@ -258,6 +264,13 @@ namespace Beagle.Daemon {
 			}
 		}
 
+		public void Rename (Uri old_uri, Uri new_uri)
+		{
+			Logger.Log.Error ("**** LuceneDriver does not support Rename!");
+			Logger.Log.Error ("**** old_uri={0}", old_uri);
+			Logger.Log.Error ("**** new_uri={0}", new_uri);
+		}
+
 		public int PendingAdds {
 			get { return pending_adds; }
 		}
@@ -334,7 +347,7 @@ namespace Beagle.Daemon {
 			IndexWriter writer = null;
 			foreach (Indexable indexable in pending_indexables) {
 				
-				Log.Debug ("+ {0}", indexable.Uri);
+				Log.Debug ("+ {0}", indexable.DisplayUri);
 				
 				Document doc = null;
 				try {
@@ -361,7 +374,7 @@ namespace Beagle.Daemon {
 
 			// Step #3: Fire off an event telling what we just did.
 			if (ChangedEvent != null) {
-				ChangedEvent (this, added_uris, removed_uris);
+				ChangedEvent (this, added_uris, removed_uris, empty_collection);
 			}
 
 			lock (pending_by_uri)
@@ -395,32 +408,9 @@ namespace Beagle.Daemon {
 			Stopwatch watch = new Stopwatch ();
 			watch.Start ();
 
-			bool optimize_in_process = true;
-
-#if false
-			if (optimizer_path != null) {
-				try {
-					// Try to optimize in an external process
-					Process p = new Process ();
-					p.StartInfo.UseShellExecute = true;
-					p.StartInfo.FileName = optimizer_path;
-					p.StartInfo.Arguments = StorePath;
-					p.Start ();
-					p.WaitForExit ();
-					optimize_in_process = false;
-				} catch (Exception ex) {
-					Log.Debug ("Caught exception while running beagle-index-optimizer");
-					Log.Debug (ex);
-				}
-			}
-#endif
-
-			if (optimize_in_process) {
-				Log.Debug ("Optimizing in-process.");
-				IndexWriter writer = new IndexWriter (Store, null, false);
-				writer.Optimize ();
-				writer.Close ();
-			}
+			IndexWriter writer = new IndexWriter (Store, null, false);
+			writer.Optimize ();
+			writer.Close ();
 
 			watch.Stop ();
 
@@ -438,8 +428,10 @@ namespace Beagle.Daemon {
 		// truncated.
 		public void DoQuery (QueryBody           body,
 				     IQueryResult        result,
-				     ICollection         list_of_uris,
+				     ICollection         search_subset, // should be internal uris
+				     ICollection         bonus_uris,    // should be internal uris
 				     UriFilter           uri_filter,
+				     UriRemapper         uri_remapper, // map to external uris
 				     RelevancyMultiplier relevancy_multiplier)
 		{
 			double t_lucene;
@@ -450,7 +442,7 @@ namespace Beagle.Daemon {
 			IndexReader reader = IndexReader.Open (Store);
 
 			LNS.Searcher searcher = new LNS.IndexSearcher (reader);
-			LNS.Query query = ToLuceneQuery (body, list_of_uris);
+			LNS.Query query = ToLuceneQuery (body, search_subset, bonus_uris);
 
 			LNS.Hits hits = searcher.Search (query);
 			sw.Stop ();
@@ -483,12 +475,13 @@ namespace Beagle.Daemon {
 				}
 
 				Hit hit = FromLuceneDocToHit (doc, hits.Id (i), score);
+				if (uri_remapper != null)
+					hit.Uri = uri_remapper (hit.Uri);
 
 				if (relevancy_multiplier != null) {
 					double m = relevancy_multiplier (hit);
 					hit.ScoreMultiplier = (float) m;
 				}
-
 				result.Add (hit);
 			}
 
@@ -501,10 +494,11 @@ namespace Beagle.Daemon {
 			// The call to searcher.Close () also closes the IndexReader.
 			searcher.Close ();
 
-			log.Debug ("{0}: n_hits={1} lucene={2:0.00}s  assembly={3:0.00}s",
+			log.Debug ("{0}: n_hits={1} lucene={2:0.00}s assembly={3:0.00}s",
 				   StorePath, n_hits, t_lucene, t_assembly);
 		}
-
+		
+		// FIXME: This should support Uri filtering, Uri remapping, etc.
 		public ICollection DoQueryByUri (ICollection list_of_uris)
 		{
 			LNS.BooleanQuery uri_query = new LNS.BooleanQuery ();
@@ -594,7 +588,7 @@ namespace Beagle.Daemon {
 			// First we add the Indexable's 'canonical' properties
 			// to the Document.
 			
-			f = Field.Keyword ("Uri", indexable.Uri.ToString ());
+			f = Field.Keyword ("Uri", UriFu.UriToSerializableString (indexable.Uri));
 			doc.Add (f);
 
 			f = Field.Keyword ("Type", indexable.Type);
@@ -651,6 +645,40 @@ namespace Beagle.Daemon {
 			
 			return doc;
 		}
+
+		static public LNS.Query ToUriQuery (ICollection list_of_uris, UriRemapper remapper)
+		{
+			if (list_of_uris == null || list_of_uris.Count == 0)
+				return null;
+
+			LNS.BooleanQuery query = new LNS.BooleanQuery ();
+			int max_clauses = LNS.BooleanQuery.GetMaxClauseCount ();
+			int clause_count = 0;
+
+			foreach (Uri original_uri in list_of_uris) {
+				Uri uri = original_uri;
+				if (remapper != null)
+					uri = remapper (uri);
+				Term term = new Term ("Uri", uri.ToString ()); // FIXME: Do we need some UriFu here?
+				LNS.Query term_query = new LNS.TermQuery (term);
+				query.Add (term_query, false, false);
+				++clause_count;
+				// If we have to many clases, nest the queries
+				if (clause_count == max_clauses) {
+					LNS.BooleanQuery new_query = new LNS.BooleanQuery ();
+					new_query.Add (query, false, false);
+					query = new_query;
+					clause_count = 1;
+				}
+			}
+
+			return query;
+		}
+
+		static public LNS.Query ToUriQuery (ICollection list_of_uris)
+		{
+			return ToUriQuery (list_of_uris, null);
+		}
 		
 		private LNS.Query ToCoreLuceneQuery (QueryBody body, string field)
 		{
@@ -712,78 +740,117 @@ namespace Beagle.Daemon {
 
 		}
 
+		// search_subset limits the score of our search to that set of Uris
+		// bonus_uris are always matched by the query
 		private LNS.Query ToLuceneQuery (QueryBody body,
-						 ICollection listOfUris)
+						 ICollection search_subset,
+						 ICollection bonus_uris)
 		{
-			LNS.BooleanQuery luceneQuery = new LNS.BooleanQuery ();
+			LNS.BooleanQuery body_query = null;
+			LNS.Query        search_subset_query = null;
+			LNS.Query        bonus_uris_query = null;
+			LNS.BooleanQuery mime_type_query = null;
+			LNS.BooleanQuery hit_type_query = null;
 			
 			if (body.Text.Count > 0) {
-				LNS.BooleanQuery contentQuery = new LNS.BooleanQuery ();
 
-				LNS.Query propTQuery;
-				propTQuery = ToCoreLuceneQuery (body, "PropertiesText");
-				if (propTQuery != null) {
-					propTQuery.SetBoost (2.5f);
-					contentQuery.Add (propTQuery, false, false);
+				body_query = new LNS.BooleanQuery ();
+
+				LNS.Query q;
+				
+				q = ToCoreLuceneQuery (body, "PropertiesText");
+				if (q != null) {
+					q.SetBoost (2.5f);
+					body_query.Add (q, false, false);
 				}
 
-				LNS.Query propKQuery;
-				propKQuery = ToCoreLuceneQuery (body, "PropertiesKeyword");
-				if (propKQuery != null) {
-					propKQuery.SetBoost (2.5f);
-					contentQuery.Add (propKQuery, false, false);
+				q = ToCoreLuceneQuery (body, "PropertiesText");
+				if (q != null) {
+					q.SetBoost (2.5f);
+					body_query.Add (q, false, false);
+				}
+
+				q = ToCoreLuceneQuery (body, "PropertiesKeyword");
+				if (q != null) {
+					q.SetBoost (2.5f);
+					body_query.Add (q, false, false);
 				}
 				
-				LNS.Query hotQuery;
-				hotQuery = ToCoreLuceneQuery (body, "HotText");
-				if (hotQuery != null) {
-					hotQuery.SetBoost (1.75f);
-					contentQuery.Add (hotQuery, false, false);		
+				q = ToCoreLuceneQuery (body, "HotText");
+				if (q != null) {
+					q.SetBoost (1.75f);
+					body_query.Add (q, false, false);		
 				}
 				
-				LNS.Query textQuery;
-				textQuery = ToCoreLuceneQuery (body, "Text");
-				if (textQuery != null) {
-					contentQuery.Add (textQuery, false, false);
+				q = ToCoreLuceneQuery (body, "Text");
+				if (q != null) {
+					body_query.Add (q, false, false);
 				}
-
-				luceneQuery.Add (contentQuery, true, false);
 			}
 
-			// If mime types are specified, we must match one of them.
+			search_subset_query = ToUriQuery (search_subset, null);
+
+			bonus_uris_query = ToUriQuery (bonus_uris, null);
+				
 			if (body.MimeTypes.Count > 0) {
-				LNS.BooleanQuery mimeTypeQuery = new LNS.BooleanQuery ();
-				foreach (string mimeType in body.MimeTypes) {
-					Term t = new Term ("MimeType", mimeType);
+				mime_type_query = new LNS.BooleanQuery ();
+				foreach (string mime_type in body.MimeTypes) {
+					Term t = new Term ("MimeType", mime_type);
 					LNS.Query q = new LNS.TermQuery (t);
-					mimeTypeQuery.Add (q, false, false);
+					mime_type_query.Add (q, false, false);
 				}
-				luceneQuery.Add (mimeTypeQuery, true, false);
 			}
 
-			// If hit types are specified, we must match one of them.
 			if (body.HasHitTypes) {
-				LNS.BooleanQuery hitTypeQuery = new LNS.BooleanQuery ();
-				foreach (string hitType in body.HitTypes) {
-					Term t = new Term ("Type", hitType);
+				hit_type_query = new LNS.BooleanQuery ();
+				foreach (string hit_type in body.HitTypes) {
+					Term t = new Term ("Type", hit_type);
 					LNS.Query q = new LNS.TermQuery (t);
-					hitTypeQuery.Add (q, false, false);
+					hit_type_query.Add (q, false, false);
 				}
-				luceneQuery.Add (hitTypeQuery, true, false);
 			}
 
-			// If a list of Uris is specified, we must match one of them.
-			if (listOfUris != null && listOfUris.Count > 0) {
-				LNS.BooleanQuery uriQuery = new LNS.BooleanQuery ();
-				foreach (Uri uri in listOfUris) {
-					Term t = new Term ("Uri", uri.ToString ());
-					LNS.Query q = new LNS.TermQuery (t);
-					uriQuery.Add (q, false, false);
-				}
-				luceneQuery.Add (uriQuery, true, false);
+			//
+			// Now we combine the various parts into one big query.
+			//
+
+			LNS.BooleanQuery total_query = new LNS.BooleanQuery ();
+
+			// If we have hit types or mime types, those must be matched
+			if (mime_type_query != null)
+				total_query.Add (mime_type_query, true, false);
+			if (hit_type_query != null)
+				total_query.Add (hit_type_query, true, false);
+
+			// We also must match the "content query":
+			// (body_query OR bonus_uris_query) AND search_subset_query
+
+			LNS.Query content_query = null;
+
+			if (body_query != null && bonus_uris_query != null) {
+				LNS.BooleanQuery q = new LNS.BooleanQuery ();
+				q.Add (body_query, false, false);
+				q.Add (bonus_uris_query, false, false);
+				content_query = q;
+			} else if (body_query != null) {
+				content_query = body_query;
+			} else if (bonus_uris_query != null) {
+				content_query = bonus_uris_query;
 			}
 
-			return luceneQuery;
+			if (content_query != null && search_subset_query != null) {
+				LNS.BooleanQuery q = new LNS.BooleanQuery ();
+				q.Add (content_query, true, false);
+				q.Add (search_subset_query, true, false);
+				content_query = q;
+			} else if (search_subset_query != null) {
+				content_query = search_subset_query;
+			}
+
+			if (content_query != null)
+				total_query.Add (content_query, true, false);
+
+			return total_query;
 		}
 		
 		static private Uri UriFromLuceneDoc (Document doc)
@@ -938,9 +1005,11 @@ namespace Beagle.Daemon {
 			{
 				Lucene.Net.Analysis.Token token;
 				while ( (token = token_stream.Next ()) != null) {
+#if false
 					if (total_count > 0 && total_count % 5000 == 0)
 						Logger.Log.Debug ("BeagleNoiseFilter filtered {0} of {1} ({2:0.0}%)",
 								  noise_count, total_count, 100.0 * noise_count / total_count);
+#endif
 					++total_count;
 					if (IsNoise (token.TermText ())) {
 						++noise_count;
